/*
 * $Id: RendererModel.java 17188 2010-05-21 19:35:20Z alexsmirnov $
 *
 * License Agreement.
 *
 * Rich Faces - Natural Ajax for Java Server Faces (JSF)
 *
 * Copyright (C) 2007 Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */



package org.richfaces.cdk.model;

import org.richfaces.cdk.templatecompiler.model.Template;
import org.richfaces.cdk.util.Strings;

/**
 * <p class="changed_added_4_0"></p>
 * @author asmirnov@exadel.com
 *
 */
public class RendererModel extends ModelElementBase implements ModelElement<RendererModel> {

    private static final long serialVersionUID = -5802466539382148578L;

    private String family;

    private String componentType;

    private String templatePath;

    private Template template;

    private boolean rendersChildren;
    
    private TagModel tag;

    public RendererModel() { }

    public RendererModel(Type type) {
        setType(type);
    }

    public Type getKey() {
        return getType();
    }

    public void setType(Type type) {
        this.setId(type);
    }

    public Type getType() {
        return (Type) getId();
    }

    @Merge
    public Template getTemplate() {
        return template;
    }

    public void setTemplate(Template template) {
        this.template = template;
    }

    @Merge
    public String getFamily() {
        return family;
    }

    public void setFamily(String family) {
        this.family = family;
    }

    public ClassName getRendererClass() {
        return getTargetClass();
    }

    public void setRendererClass(ClassName rendererClass) {
        setTargetClass(rendererClass);
    }

    public boolean isRendersChildren() {
        return this.rendersChildren;
    }

    public void setRendersChildren(boolean rendersChildren) {
        this.rendersChildren = rendersChildren;
    }

    public TagModel getTag() {
        return this.tag;
    }

    public void setTag(TagModel tag) {
        this.tag = tag;
    }

    public String getTemplatePath() {
        return templatePath;
    }

    public void setTemplatePath(String templatePath) {
        this.templatePath = templatePath;
    }

    public <R,D> R accept(Visitor<R,D> visitor, D data) {
        return visitor.visitRender(this,data);
    }

    @Override
    public void merge(RendererModel other) {
        if (other == null) {
            return;
        }
        
        ComponentLibrary.merge(this, other);

        // TODO review
        ClassName targetClass = this.getTargetClass();
        if (targetClass == null || Strings.isEmpty(targetClass.getName())) {
            setTargetClass(other.getTargetClass());
        }
        
        ClassName baseClass = this.getBaseClass();
        if (baseClass == null || Strings.isEmpty(baseClass.getName())) {
            setTargetClass(other.getBaseClass());
        }
    }

    @Override
    public boolean same(RendererModel other) {
        if (null != getType() && null != other.getType()) {
            // compare families ?
            if (null != getFamily() && null != other.getFamily() && !getFamily().equals(other.getFamily())) {
                return false;
            }
            // Both types not null, compare them.
            return getType().equals(other.getType());
        } 
        // one or both types are null, compare classes.
        if (null != getRendererClass() && getRendererClass().equals(other.getRendererClass())) {
            return true;
        }
        return false;
    }

    public String getComponentType() {
        return componentType;
    }

    public void setComponentType(String componentType) {
        this.componentType = componentType;
    }
    
    @Override
    protected PropertyBase createAttribute() {
        return new AttributeModel();
    }

    /**
     * <p class="changed_added_4_0">FacesId for lookup renderer in the model.</p>
     * @author asmirnov@exadel.com
     *
     */
    public static class Type extends FacesId {
        
        private static final long serialVersionUID = -7819560321446149642L;

        /**
         * <p class="changed_added_4_0"></p>
         * TODO - use family as part of key ?
         * @param type
         */
        public Type(String type) {
            super(type);
        }
    }

    @Override
    public String toString() {
        return "Renderer {type: " + getType() + ", family: " + getFamily() + "}";
    }
}
