/*
 * $Id: CompositeInterface.java 16714 2010-04-03 00:23:10Z alexsmirnov $
 *
 * License Agreement.
 *
 * Rich Faces - Natural Ajax for Java Server Faces (JSF)
 *
 * Copyright (C) 2007 Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.cdk.templatecompiler.model;

import java.io.Serializable;
import java.util.List;

import javax.faces.render.RenderKitFactory;
import javax.faces.render.Renderer;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * <p class="changed_added_4_0"></p>
 *
 * @author asmirnov@exadel.com
 */
@XmlRootElement(name = "interface", namespace = Template.COMPOSITE_NAMESPACE)
public class CompositeInterface implements Serializable {

    private static final long serialVersionUID = -5578359507253872500L;

    private String componentFamily;

    private List<Attribute> attributes;

    private List<ResourceDependency> resourceDependencies;

    private List<ImportAttributes> attributesImports;
    
    private String renderKitId = RenderKitFactory.HTML_BASIC_RENDER_KIT;

    private String javaClass;

    private String baseClass = Renderer.class.getName();

    private String rendererType;

    private Boolean rendersChildren = null;

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @return the family
     */
    @XmlElement(name = "component-family", namespace = Template.CDK_NAMESPACE)
    public String getComponentFamily() {
        return this.componentFamily;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @param family the family to set
     */
    public void setComponentFamily(String family) {
        this.componentFamily = family;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @return the attributes
     */
    @XmlElement(name = "attribute", namespace = Template.COMPOSITE_NAMESPACE)
    public List<Attribute> getAttributes() {
        return this.attributes;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @param attributes the attributes to set
     */
    public void setAttributes(List<Attribute> attributes) {
        this.attributes = attributes;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @return the resourceDependencies
     */
    @XmlElementWrapper(name = "resource-dependencies", namespace = Template.CDK_NAMESPACE)
    @XmlElement(name = "resource-dependency", namespace = Template.CDK_NAMESPACE)
    public List<ResourceDependency> getResourceDependencies() {
        return resourceDependencies;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @param resourceDependencies the resourceDependencies to set
     */
    public void setResourceDependencies(
        List<ResourceDependency> resourceDependencies) {
        this.resourceDependencies = resourceDependencies;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @return the renderKitId
     */
    @XmlElement(name = "renderkit-id", namespace = Template.CDK_NAMESPACE)
    public String getRenderKitId() {
        return this.renderKitId;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @param renderKitId the renderKitId to set
     */
    public void setRenderKitId(String renderKitId) {
        this.renderKitId = renderKitId;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @return the javaClass
     */
    @XmlElement(name = "class", namespace = Template.CDK_NAMESPACE)
    public String getJavaClass() {
        return this.javaClass;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @param javaClass the javaClass to set
     */
    public void setJavaClass(String javaClass) {
        this.javaClass = javaClass;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @return the rendererType
     */
    @XmlElement(name = "renderer-type", namespace = Template.CDK_NAMESPACE)
    public String getRendererType() {
        return this.rendererType;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @param rendererType the rendererType to set
     */
    public void setRendererType(String rendererType) {
        this.rendererType = rendererType;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @return the baseClass
     */
    @XmlElement(name = "superclass", namespace = Template.CDK_NAMESPACE)
    public String getBaseClass() {
        return this.baseClass;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @param baseClass the baseClass to set
     */
    public void setBaseClass(String baseClass) {
        this.baseClass = baseClass;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @return the rendersChildren
     */
    @XmlElement(name = "renders-children", namespace = Template.CDK_NAMESPACE)
    public Boolean getRendersChildren() {
        return rendersChildren;
    }

    /**
     * @param rendersChildren the rendersChildren to set
     */
    public void setRendersChildren(Boolean rendersChildren) {
        this.rendersChildren = rendersChildren;
    }
    
    /**
     * <p class="changed_added_4_0"></p>
     *
     * @return the attributesImports
     */
    @XmlElement(name = "import-attributes", namespace = Template.CDK_NAMESPACE)
    public List<ImportAttributes> getAttributesImports() {
        return attributesImports;
    }

    /**
     * <p class="changed_added_4_0"></p>
     *
     * @param attributesImports the attributesImports to set
     */
    public void setAttributesImports(List<ImportAttributes> attributesImports) {
        this.attributesImports = attributesImports;
    }
    
}
